# preferences.tcl --
#
# This file controls the preferences window. There is just one preferences
# window, if the user requests a new preferences window when the window
# already exists, it is just unmapped and raised (if possible).
#
#
#  TkRat software and its included text is Copyright 1996-2004 by
#  Martin Forssn
#
#  The full text of the legal notice is contained in the file called
#  COPYRIGHT, included with this distribution.

# Preferences --
#
# Make sure the user sees an preferences window.
#
# Arguments:

proc Preferences {} {
    if {![winfo exists .pref]} {
	BuildPreferences
    } else {
	wm deiconify .pref
    }
}

# Images
set checkmark_img [image create photo -data {
    R0lGODlhDwAPAIQcADSNHjWOHzeQIjmRIzuTJj6VKT+WKkGXLEKYLUicM0ue
    Nk6gOV+sTGqzWG61XHy+bH2/boHBconFeovGfJzQkKTUmafVm6nWnqvXoLPc
    qbzgs8Xkvf///////////////yH5BAEKAB8ALAAAAAAPAA8AAAUv4CeOZGme
    aKp+E7R+EfYuzHs0r5Og2VB9EsIGpUEEPoKH6gIoGF4KgOU1obyu2BAAOw==}]
set error_img [image create photo -data {
    R0lGODlhDwAPAMZAALgEALoGCcUBDMUFAM8ABNEAAKcUFtEBFNsAAtsADd4B
    AL4NF6UaItwAGNsAKekAA84QJcMoI8MnMtEgPr5BULxMU7xOXLNXUs5MStdF
    WOBFTeJDU+VDWvBBSPg9VrhhcNBdYOhSYNpbb/xidsaAhvFyj99/feh+ktmJ
    jf55gr+XjfCChbqenNqSrt2WosOrtdmps/+kpPatoPmvtfq7t/jQy9fv09z7
    6uP/+fH/6v/1/O3/+/758f/76/z+6v7+/v//////////////////////////
    ////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////yH5BAEKAEAA
    LAAAAAAPAA8AAAfCgECCgh0BAwcYg4otFgwJCwILDRAiixkLAw0yMTQeCA0U
    gxaYBQs5PzgmAAMCFUAqEAUICAGnO6oFCg0nGAmzBQc8Oj8jCAUEAiEQBwUD
    AAc1OzkbCM4BEQerCAIJM9IbCc8FBRG+AAQHKTk+sQAPCREhBQ8AshwlJAIE
    A7QXQAYPBhAIYGMHjxQAzi1wAeRDtQcFfKBKMS+BKyAvJgQgkKDHjRsrCBCQ
    oAhGBQgKPITI4ODARUVAWIAwMACCBhSKAgEAOw==}]


# BuildPreferences --
#
# Builds the preferences window.
#
# Arguments:

proc BuildPreferences {} {
    global t b pref option

    # Initialize data table
    foreach l [GetLanguages] {
	lappend lang [lrange $l 0 1]
    }

    # XXX
    set spell_dictionaries [list [list auto $t(auto)]]
    set pref(dictionaries) {}

    # The lists here can have the following elements
    # option var label_id value_list
    # checkbutton var label_id onvalue offvalue
    # bool var label_id
    # entry var label_id
    # entry_unit var label_id unit
    # special var label_id special_proc
    # label label_text anchor
    # custom custom_proc
    set pref(appearance) "\
	{option language language [list $lang]} \
	{entry charset charset} \
	{bool useinputmethods useinputmethods} \
    "
    set pref(appearance,graphics) "\
	{special color_set color_scheme SetupColor} \
	{label $t(fonts) center} \
	{special prop_big prop_big {SelectFont prop_big}} \
	{special prop_norm prop_norm {SelectFont prop_norm}} \
	{special prop_light prop_light {SelectFont prop_light}} \
	{special fixed_norm fixed_norm {SelectFont fixed_norm}} \
	{special fixed_bold fixed_bold {SelectFont fixed_bold}} \
	{special fixed_italic fixed_italic {SelectFont fixed_italic}} \
	{special watcher_font watcher_font {SelectFont watcher_font}} \
        {custom SetupFontSizeButtons} \
    "
    set pref(appearance,msglist) "\
	{entry list_format list_format} \
	{option folder_sort sort_order \
		{{threaded {$t(sort_threaded)}} \
		 {subject {$t(sort_subject)}} \
		 {subjectonly {$t(sort_subjectonly)}} \
		 {sender {$t(sort_sender)}} \
		 {folder {$t(sort_folder)}} \
		 {reverseFolder {$t(sort_reverseFolder)}} \
		 {date {$t(sort_date)}} \
		 {reverseDate {$t(sort_reverseDate)}} \
		 {size {$t(sort_size)}}\
		 {reverseSize {$t(sort_reverseSize)}}}} \
	{option start_selection start_selection \
		{{first {$t(first_message)}} \
		 {last {$t(last_message)}} \
		 {first_new {$t(first_new_message)}} \
		 {before_new {$t(before_first_new_message)}}}} \
	{option browse default_browse_mode \
		{{normal {$t(no_browse)}} \
		 {browse {$t(do_browse)}} \
		 {folder {$t(use_folder_default)}}}} \
	{option show_header show_headers \
		{{all {$t(show_all_headers)}} \
		 {selected {$t(show_selected_headers)}} \
		 {none {$t(show_no_headers)}}}} \
	{entry show_header_selection show_header_selection} \
        {bool prefer_other_over_html avoid_html} \
    "
    # Don't show HTML preferences if the package can't be loaded
    if {![catch {package require http}]} {
    set pref(appearance,html) "\
	{special html_prop_font html_prop_font {SelectFont html_prop_font}} \
	{special html_fixed_font html_fixed_font \
	        {SelectFont html_fixed_font}} \
	{entry html_prop_font_sizes html_prop_font_sizes} \
	{entry html_fixed_font_sizes html_fixed_font_sizes} \
	{bool html_show_images html_show_images} \
        {entry html_min_image_size html_min_image_size} \
        {entry html_proxy_host html_proxy_host} \
        {entry html_proxy_port html_proxy_port} \
        {entry html_timeout html_timeout} \
        "
    }
    set pref(network) "\
	{message {} {$t(start_online_mode)}} \
	{option start_online_mode start_mode \
	    {{last {$t(som_last)}} \
	     {online {$t(som_online)}} \
             {offline {$t(som_offline)}}}} \
    "
    set pref(network,www) "\
	{option url_viewer url_viewer \
		{{RatUP {$t(userproc): RatUP_ShowURL}} \
		 {mozilla Mozilla} \
		 {galeon Galeon} \
		 {netscape Netscape} \
		 {opera Opera} \
		 {lynx Lynx}
                 {other $t(other)}}} \
	{option url_behavior show_url_in \
                 {{old_window {$t(reuse_old_window)}} \
                  {new_window {$t(new_window)}} \
                  {new_tab {$t(new_tab)}}}} \
	{entry mozilla mozilla_cmd} \
	{entry galeon galeon_cmd} \
	{entry netscape netscape_cmd} \
	{entry opera opera_cmd} \
	{entry lynx lynx_cmd} \
	{entry other_browser other_browser_cmd} \
    "
    set pref(replies) "\
	{bool wrap_cited wrap_cited} \
	{bool skip_sig skip_sig} \
        {bool append_sig append_sig} \
        {bool reply_bottom reply_bottom} \
    "
    set pref(spell_checking) "\
	{bool autospell mark_misspellings} \
	{option def_spell_dict dictionary \
                {$spell_dictionaries}} \
        {special auto_dicts auto_dicts SetupCheckDicts} \
	{special spell_path spell_cmd SetupSpellPath} \
    "
    set pref(roles) "\
	{entry name role_name} \
	{entry signature signature_file} \
	{special save_outgoing save_out SetupDefaultSave} \
	{custom SetupRole} \
    "
    set pref(roles,address) "\
	{entry from use_from_address} \
	{entry bcc default_bcc} \
	{entry reply_to default_reply_to} \
	{message $t(tip) {$t(test_by)}} \
	{entry pgp_keyid pgp_keyid} \
    "
    set pref(roles,sending) "\
	{option sendprot sendprot \
		{{smtp {$t(smtp)}} {prog {$t(user_program)}}}} \
	{entry smtp_hosts smtp_hosts} \
	{entry smtp_user smtp_user} \
	{entry smtp_passwd passwd} \
        {bool validate_cert ssl_check_cert} \
	{entry sendprog sendprog} \
	{bool sendprog_8bit sendprog_8bit} \
        {bool same_sending_prefs same_sending_prefs} \
    "
    set pref(roles,advanced) "\
        {label {$t(normally_ok)} center} \
        {label {$t(unqual_adr_domain)} w} \
        {entry uqa_domain domain} \
        {label {$t(smtp_from_long)} w} \
        {entry smtp_helo host}
    "
    set pref(roles,pgp) "\
	{bool sign_outgoing sign_outgoing} \
	{special sign_as sign_as PrefSetupSignAs} \
    "
    set pref(advanced) "\
	{entry main_window_name window_name} \
	{entry icon_name icon_name} \
	{option icon icon_bitmap \
		{{normal {$t(normal_bitmap)}} \
		 {small {$t(small_bitmap)}} \
		 {none {$t(none)}}}} \
    "
    set pref(advanced,behaviour) "\
	{bool iconic startup_iconic} \
	{bool info_changes show_changes_at_start} \
	{bool mail_steal check_stolen_mail} \
	{bool expunge_on_close expunge_on_close} \
	{bool keep_pos remember_pos} \
	{bool checkpoint_on_unmap checkpoint_on_unmap} \
	{entry_unit checkpoint_interval checkpoint_interval {$t(seconds)}} \
	{option print_header print_headers \
		{{all {$t(all)}} {selected {$t(selected)}} {none {$t(none)}}}} \
    "
    set pref(advanced,appearance) "\
	{bool override_color override_color} \
	{bool override_fonts override_fonts} \
	{bool tearoff menu_tearoff} \
	{entry_unit log_timeout log_timeout {$t(seconds)}} \
    "
    set pref(advanced,ssh) "\
	{entry ssh_path ssh_path} \
	{entry_unit ssh_timeout ssh_timeout {$t(seconds)}} \
    "
    set pref(advanced,files) "\
	{entry tmp tmp_dir} \
	{entry userproc userproc_file} \
	{entry mailcap_path mailcap_path} \
	{entry print_command print_command} \
	{entry terminal terminal_command} \
	{entry debug_file debug_file} \
	{entry mimeprog mimeprog} \
    "
    set pref(advanced,caching) "\
	{bool cache_passwd cache_passwd} \
	{entry_unit cache_passwd_timeout cache_timeout {$t(seconds)}} \
	{bool cache_conn cache_conn} \
	{entry_unit cache_conn_timeout cache_timeout {$t(seconds)}} \
    "
    set pref(advanced,folder_dynamic) "\
	{option dynamic_behaviour dynamic_behaviour \
		{{expanded {$t(dyn_expanded)}} \
		 {closed {$t(dyn_closed)}}}} \
    "
    set pref(advanced,network) "\
	{entry domain domain} \
	{entry remote_user remote_user} \
	{entry remote_host remote_host} \
	{entry imap_port imap_port} \
	{entry pop3_port pop3_port} \
	{entry urlprot url_protocols} \
	{entry url_color url_color} \
    "
    set pref(advanced,dbase) "\
	{option def_extype extype \
		{{none {$t(none)}} \
		 {remove {$t(remove)}} \
		 {incoming {$t(incoming)}} \
		 {backup {$t(backup)}}}} \
	{entry_unit def_exdate exdate {$t(days)}} \
	{entry dbase_backup dbase_backup} \
	{entry_unit chunksize chunksize {$t(messages)}} \
	{entry_unit expire_interval expire_interval {$t(days)}} \
    "
    set pref(advanced,watcher) "\
	{entry_unit watcher_time intervals {$t(seconds)}} \
	{entry_unit watcher_max_height max_height {$t(lines)}} \
	{option watcher_show show_messages \
		{{new {$t(new)}} {all {$t(all)}}}} \
	{entry watcher_format list_format} \
	{entry watcher_bell bell_ringings} \
	{entry watcher_title watcher_title} \
    "
    set pref(advanced,compose) "\
	{entry compose_headers headers} \
	{entry_unit wrap_length wrap_length {$t(characters)}} \
        {entry_unit compose_backup compose_backup {$t(seconds)}} \
	{bool sigdelimit sigdelimit} \
	{bool lookup_name lookup_name} \
	{bool always_editor always_use_external_editor} \
    "
    set pref(advanced,replies) "\
	{entry re_regexp reply_regexp} \
	{entry attribution attribution} \
	{entry forwarded_message forwarded_label} \
	{entry no_subject no_subject} \
	{entry reply_lead reply_lead} \
	{entry citexp citexp} \
    "
    set pref(advanced,sending) "\
	{bool create_sender create_sender} \
	{entry_unit smtp_timeout smtp_timeout {$t(seconds)}} \
	{bool smtp_reuse smtp_reuse} \
	{option smtp_verbose smtpv \
		{{0 {$t(none)}} \
		 {1 {$t(terse)}} \
		 {2 {$t(normal)}} \
		 {3 {$t(verbose)}}}} \
	{bool force_send force_send} \
        {option parm_enc parm_enc_mode \
                {{rfc2047 {$t(enc_rfc2047)}}
                 {rfc2231 {$t(enc_rfc2231)}}
                 {both {$t(enc_both)}}}} \
    "
    set pref(advanced,pgp) "\
	{option pgp_version pgp_version \
		{{0 $t(none)} {gpg-1 {GPG-1}} {2 {PGP-2}} \
		 {5 {PGP-5}} {6 {PGP-6}} {auto {$t(auto)}}}} \
	{entry pgp_path pgp_path} \
	{entry pgp_args pgp_extra_args} \
	{entry pgp_keyring pgp_keyring} \
	{bool cache_pgp cache_passwd} \
	{entry_unit cache_pgp_timeout cache_timeout {$t(seconds)}} \
	{bool pgp_encrypt encrypt_outgoing} \
    "

    # Create top window structures
    set w .pref
    toplevel $w -class TkRat
    wm title $w $t(preferences)
    set pref(scrollpane) $w.pane
    set pref(pane) [rat_scrollframe::create $w.pane -bd 2 -relief flat]
    Size $w.pane prefPane
    set pref(lastPref) ""
    set pref(selected) ""

    # Setup tree
    set pref(tree) [rat_tree::create $w.tree \
	    -sizeid prefTree -selectcallback PrefSelectPane]
    $pref(tree) autoredraw 0
    set topnode [$pref(tree) gettopnode]
    set pref(rnode) \
	    [$topnode add folder -label $t(roles) -state open -id rnode]
    PrefPopulateRoles
    set onode [$topnode add folder -label $t(options) -state open]
    foreach n {{appearance {graphics msglist html}}
               {network {www}}
               replies spell_checking
               {advanced {behaviour appearance ssh files caching
                          folder_dynamic network dbase watcher compose
	                  replies sending pgp}}} {
	set n0 [lindex $n 0]
	set name $t($n0)
	if {1 != [llength $n]} {
	    set node [$onode add folder -label $name -state closed \
		    -id [list option $n0]]
	    foreach n1 [lindex $n 1] {
		$node add leaf -label $t($n1) -id [list option $n0,$n1]
	    }
	} else {
	    $onode add leaf -label $name -id [list option $n0]
	}
    }
    $pref(tree) redraw

    # The buttons
    frame $w.buttons
    button $w.buttons.ok -text $t(apply) -command "PrefApply $w" \
	    -default active -state disabled
    button $w.buttons.reset -text $t(reset) -state disabled \
	    -command {
			global pref
			foreach v $pref(vars,$pref(lastPref)) {
			    set pref(opt,$v) $pref(old,$v)
			}
			foreach but $pref(traceButtons) {
			    $but configure -state disabled
			}
		     }
    button $w.buttons.close -text $t(close) \
	    -command "PrefCheck $w; \
		      RecordPos $w preferences; \
		      RecordSize $w.pane prefPane; \
		      wm withdraw $w"
    pack $w.buttons.ok \
	 $w.buttons.reset \
         $w.buttons.close -side left -expand 1
    bind $w <Return> "PrefApply $w"
    set b($w.buttons.ok) apply_prefs
    set b($w.buttons.reset) reset_changes
    set b($w.buttons.close) dismiss
    set pref(traceButtons) [list $w.buttons.ok $w.buttons.reset]

    grid $w.tree $w.pane -sticky nwse
    grid ^ $w.buttons  -sticky ew
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
    grid columnconfigure $w 1 -weight 5
    grid propagate $w.pane 0

    Place $w preferences

    # Initialize tracing function
    set pref(traceChanges) 0
    trace variable pref w PrefTraceProc
}

# PrefSelectPane --
#
# Called when the user selects an item in the tree
#
# Arguments:
# pane - pane to fill in

proc PrefSelectPane {pane} {
    global pref

    if {"" == $pane} {
	return fail
    }
    PrefCheck $pref(pane)
    set pref(selected) $pane
    foreach c [winfo children $pref(pane)] {
	destroy $c
    }
    switch [lindex $pane 0] {
	option {
	    PrefBuild [lindex $pane 1] $pref(pane) ""
	}
	role {
	    PrefBuild [lindex $pane 2] $pref(pane) "[lindex $pane 1],"
	}
	rnode {
	    PrefBuildRnode $pref(pane)
	}
    }
    rat_scrollframe::recalc $pref(scrollpane)
    return ok
}

# PrefBuild --
#
# Build a preferences pane
#
# Arguments:
# pane - Which pane of preferences to build
# w    - Name of frame to build it into
# rp   - Role prefix

proc PrefBuild {pane w rp} {
    global t b option pref

    set pref(traceChanges) 0
    set row 0
    set pref(lastPref) $pane
    set pref(rolePrefix) $rp
    set pref(vars,$pane) {}
    foreach p $pref($pane) {
	grid rowconfigure $w $row -weight 0

	if {![regexp {^(label|message|custom)$} [lindex $p 0]]} {
	    set var "${rp}[lindex $p 1]"
	    set bvar [lindex $p 1]
	    lappend pref(vars,$pane) $var
	    if {[info exists option($var)]} {
		set pref(opt,$var) $option($var)
	    } else {
		set pref(opt,$var) {}
	    }
	}
	switch [lindex $p 0] {
	    entry {
		label $w.r${row}_lab -text $t([lindex $p 2]):
		grid $w.r${row}_lab -row $row -sticky ne -pady 1
		entry $w.r${row}_item -textvariable pref(opt,$var)
		grid $w.r${row}_item - -row $row -column 1 -sticky we
		set b($w.r${row}_item) pref_$bvar
	    }
	    entry_unit {
		label $w.r${row}_lab -text $t([lindex $p 2]):
		grid $w.r${row}_lab -row $row -sticky ne -pady 1
		entry $w.r${row}_item -textvariable pref(opt,$var)
		label $w.r${row}_unit -text ([lindex $p 3])
		grid $w.r${row}_item -row $row -column 1 -sticky we
		grid $w.r${row}_unit -row $row -column 2 -sticky w
		set b($w.r${row}_item) pref_$bvar
	    }
	    option {
		label $w.r${row}_lab -text $t([lindex $p 2]):
		grid $w.r${row}_lab -row $row -sticky ne -pady 1
		OptionMenu $w.r${row}_item $var [lindex $p 3]
		grid $w.r${row}_item - -row $row -column 1 -sticky w
		set b($w.r${row}_item) pref_$bvar
	    }
	    checkbutton {
		checkbutton $w.r${row}_cb -text $t([lindex $p 2]) \
		    -variable pref(opt,$var) -onvalue [lindex $p 3] \
		    -offvalue [lindex $p 4]
		grid $w.r${row}_cb - -row $row -column 1 -sticky w
		set b($w.r${row}_cb) pref_$bvar
	    }
	    bool {
		checkbutton $w.r${row}_cb -text $t([lindex $p 2]) \
		    -variable pref(opt,$var)
		if {$pref(opt,$var)} {
		    set pref(opt,$var) 1
		} else {
                    set pref(opt,$var) 0
		}
		grid $w.r${row}_cb - -row $row -column 1 -sticky w
		set b($w.r${row}_cb) pref_$bvar
	    }
	    special {
		label $w.r${row}_lab -text $t([lindex $p 2]):
		grid $w.r${row}_lab -row $row -sticky ne -pady 1
		eval "[lindex $p 3] $w.r${row}_item"
		grid $w.r${row}_item - -row $row -column 1 -sticky we
	    }
	    label {
		frame $w.r${row}_lab
		grid $w.r${row}_lab -row $row -columnspan 2 -sticky new -pady 1
		rat_flowmsg::create $w.r${row}_lab.l -text [lindex $p 1] \
		    -anchor [lindex $p 2] -padx 0
		pack $w.r${row}_lab.l -fill both -expand 1
	    }
	    message {
		label $w.r${row}_lab -text [lindex $p 1]
		frame $w.r${row}_message
		grid $w.r${row}_lab -row $row -sticky ne -pady 1
		grid $w.r${row}_message -row $row -column 1 -sticky we
		rat_flowmsg::create $w.r${row}_message.m \
		    -text [lindex $p 2] -padx 0 -anchor w
		pack $w.r${row}_message.m -fill both -expand 1
	    }
	    custom {
		eval "[lindex $p 1] $w.r${row}_item"
		grid $w.r${row}_item - -row $row -column 1 -sticky we
	    }
	    default {puts "Internal error <$p>"}
	}
	if {![regexp {^(label|message|custom)$} [lindex $p 0]]} {
	    set pref(old,$var) $pref(opt,$var)
	    if {[RatIsLocked option([lindex $p 1])]} {
		$w.r${row}_item configure -state disabled -relief flat
	    }
	}

	incr row
    }
    frame $w.space
    grid $w.space -row $row
    grid rowconfigure $w $row -weight 1
    grid columnconfigure $w 1 -weight 1
    foreach but $pref(traceButtons) {
	$but configure -state disabled
    }
    set pref(traceChanges) 1
}

# PrefBuildRnode --
#
# Build the roles pane
#
# Arguments:
# w    - Name of frame to build it into

proc PrefBuildRnode {w} {
    global t b

    grid rowconfigure $w 0 -weight 0
    grid rowconfigure $w 1 -weight 0
    message $w.rnmsg -text $t(roles_expl) -aspect 700
    button $w.nrole -text $t(create_new_role) -command PrefCreateRole
    grid $w.rnmsg - -row 0 -sticky ew -pady 5
    grid x $w.nrole -sticky w
}

# PrefApply --
#
# Applies any changes to the preferences made in the current window.
#
# Arguments:
# parent -	Parent window

proc PrefApply {parent} {
    global option pref t folderWindowList tk_version

    set hasChanged 0
    set needRestart 0
    set rp $pref(rolePrefix)
    switch $pref(lastPref) {
	appearance {
	    if {[string compare $option(useinputmethods) \
		    $pref(opt,useinputmethods)] && 8.3 <= $tk_version} {
		tk useinputmethods $pref(opt,useinputmethods)
	    }
	    if {[string compare $option(charset) $pref(opt,charset)]} {
		set option(charset_candidates) \
		    [linsert $option(charsets) 1 $pref(opt,charset)]
	    }
	}
	advanced,files {
	    if {![regexp %p $pref(opt,print_command)]} {
		Popup $t(no_pp_in_print_command) $parent
	    }
	    if {[string compare $option(mailcap_path) \
		    $pref(opt,mailcap_path)]} {
		set option(mailcap_path) $pref(opt,mailcap_path)
		RatMailcapReload
		set hasChanged 1
	    }
	}
	advanced,replies {
	    if {[catch {regexp -nocase $pref(opt,re_regexp) ""} e]} {
		Popup "$t(re_regexp_error): $e" $parent
		return
	    }
	    if {[catch {regexp -nocase $pref(opt,citexp) ""} e]} {
		Popup [format $t(illegal_regexp): $e] $parent
		return
	    }
	}
	roles {
	    if {[string compare $option(${rp}signature) \
		    $pref(opt,${rp}signature)]
	        && 1 == [llength [info commands RatUP_Signature]]} {
		Popup $t(sig_cmd_takes_precedence) $parent
	    }
	    if {[string compare $option(${rp}name) $pref(opt,${rp}name)]} {
		set option(${rp}name) $pref(opt,${rp}name)
		PrefSortRoles
		PrefPopulateRoles
		$pref(tree) redraw
		set hasChanged 1
	    }
	}
	roles,sending {
	    if {[string compare $option(${rp}sendprog) \
		     $pref(opt,${rp}sendprog)]
		&& ![file executable [lindex $pref(opt,${rp}sendprog) 0]]} {
		Popup $t(warning_sendprog) $parent
	    }
	    if {$pref(opt,${rp}same_sending_prefs) != 
		$option(${rp}same_sending_prefs)} {
		if {1 == $pref(opt,${rp}same_sending_prefs)} {
		    if [CheckSameSendingPrefs $rp] {
			set pref(opt,${rp}same_sending_prefs) 0
			return
		    }
		}
		set hasChanged 1
		foreach r $option(roles) {
		    set pref(opt,${r},same_sending_prefs) \
			$pref(opt,${rp}same_sending_prefs)
		}
	    }
	}
	advanced {
	    if {[string compare $option(icon) $pref(opt,icon)]} {
		SetIcon . $pref(opt,icon)
	    }
	}
	advanced,network {
	    if {[string compare $option(url_color) $pref(opt,url_color)]} {
		foreach fw [array names folderWindowList] {
		    upvar #0 $fw fh
		    $fh(text) tag configure URL \
			    -foreground $pref(opt,url_color)
		}
	    }
	}
    appearance,html {
        if {[string compare $option(html_proxy_host) $pref(opt,html_proxy_host)]} {
            ::http::config -proxyhost $pref(opt,html_proxy_host)
            if {![string length $pref(opt,html_proxy_host)]} {
                set pref(opt,html_proxy_port) ""
            }
        }
        if {[string compare $option(html_proxy_port) $pref(opt,html_proxy_port)]} {
            ::http::config -proxyport $pref(opt,html_proxy_port)
        }
    }
    }
    foreach prefs [array names pref opt,*] {
	set opt [string range $prefs 4 end]
	if {![info exists option($opt)]} {
	    continue
	}
	if {[string compare $option($opt) $pref(opt,$opt)]} {
	    set option($opt) $pref(opt,$opt)
	    set hasChanged 1
	    if { -1 != [lsearch -exact {language charset fontsize
		    			main_window_name icon_name
					default_folder pgp_enable
					override_fonts prop_norm prop_light
					fixed_norm fixed_bold watcher_font
	                                charset prop_big fixed_italic
                                        color_set} \
				$opt]} {
		set needRestart 1
	    }
	}
    }

    if {$hasChanged} {
	switch $pref(lastPref) {
	    appearance,msglist {
		foreach f [array names folderWindowList] {
		    Sync $f update
		}
	    }
	    advanced,pgp {
		InitPgp
		foreach v {pgp_version pgp_path pgp_keyring} {
		    set pref(opt,$v) $option($v)
		}
	    }
	    roles,sending {
		if {1 == $option(${rp}same_sending_prefs)} {
		    foreach r $option(roles) {
			foreach v {sendprot smtp_hosts validate_cert
			    sendprog sendprog_8bit smtp_user smtp_passwd} {
			set option(${r},$v) $option(${rp}$v)
			set pref(opt,${r},$v) $option(${rp}$v)
		    }
		}
	    }
	}
	}

	SaveOptions
    }
    if {$needRestart} {
	Popup $t(need_restart) $parent
    }

    foreach but $pref(traceButtons) {
	$but configure -state disabled
    }
}


# PrefCheck --
#
# Checks if there are any unapplied changes and if there is the user is
# queried if he wants to apply them.
#
# Arguments:
# parent - Parent of window

proc PrefCheck {parent} {
    global option pref t

    foreach prefs [array names pref opt,*] {
	set opt [string range $prefs 4 end]
	if {![info exists option($opt)]} {
	    continue
	}
	if {[string compare $option($opt) $pref(opt,$opt)]} {
	    set value [RatDialog $parent $t(unapplied_changes_title) \
		    $t(unapplied_changes) {} 0 $t(apply) $t(discard)]
	    if { 0 == $value } {
		PrefApply $parent
	    } else {
		foreach n [array names pref opt,*] {
		    unset pref($n)
		}
	    }
	    return
	}
    }
}


# OptionMenu --
#
# Generates an option menu. The generated menu will have window name "w"
# and will set the "varName" variable. The different options are
# controlled by the value arguments. Each value argument is a list of
# two elements. The first is the value to set "varName" to and the second
# is the text to show. The menubutton will use "textVar" as the textvariable.
#
# Arguments:
# w	  -	Name of menubutton to create
# varid -	Variable to set to value
# values  -	A list of lists which describes the values of this button

proc OptionMenu {w varid values} {
    upvar \#0 pref(opt,$varid) var
    global pref

    menubutton $w -textvariable pref(text,$varid) -indicatoron 1 \
		  -relief raised -menu $w.m -pady 1
    set pref(w,$varid) $w
    menu $w.m -tearoff 0
    PrefPopulateOptionsMenu $varid $values

    trace variable var w "PrefTraceOptionProc $varid"
}

proc PrefPopulateOptionsMenu {varid values} {
    upvar \#0 pref(opt,$varid) var
    upvar \#0 pref(text,$varid) text
    global pref

    set pref(values,$varid) $values
    $pref(w,$varid).m delete 0 end
    set width 10
    set text ""
    foreach elem $values {
	if {![string compare [lindex $elem 0] $var]} {
	    set text [lindex $elem 1]
	}
	$pref(w,$varid).m add command -label [lindex $elem 1] \
            -command "set pref(opt,$varid) [list [lindex $elem 0]]"
	if { $width < [string length [lindex $elem 1]]} {
	    set width [string length [lindex $elem 1]]
	}
    }
    if {"" == $text && 0 < [llength $values]} {
        set text [lindex [lindex $values 0] 1]
    }
    $pref(w,$varid) configure -width $width
}

proc PrefTraceOptionProc {varid args} {
    upvar \#0 pref(opt,$varid) var
    upvar \#0 pref(text,$varid) text
    upvar \#0 pref(values,$varid) values

    foreach v $values {
	if {![string compare $var [lindex $v 0]]} {
	    set text [lindex $v 1]
	    return
	}
    }
}

proc SetupColor {w} {
    global t option pref tk_version b

    frame $w
    menubutton $w.mb -textvariable pref(text,color_set) \
	    -indicatoron 1 -relief raised -menu $w.mb.m -pady 1
    set b($w) pref_color_scheme
    set b($w.mb) pref_color_scheme
    menu $w.mb.m -tearoff 0
    set width 20
    foreach c {
        {\#dde3eb black white black}
        {PeachPuff2 black white black}
        {SlateBlue1 black white black}
        {SteelBlue4 white LightBlue black}
        {SkyBlue1 black white black}
        {aquamarine2 black white black}
        {SpringGreen4 black PaleGreen black}
        {gray85 black gray85 black}} {
	set name $t([lindex $c 0])
	if {![string compare $c $option(color_set)]} {
	    set pref(text,color_set) $name
	}
	$w.mb.m add command -label $name \
		-command "set pref(opt,color_set) [list $c]; \
		set pref(text,color_set) [list $name]" \
		-background [lindex $c 0] -foreground [lindex $c 1]
	if { $width < [string length $name]} {
	    set width [string length $name]
	}
    }
    $w.mb configure -width $width
    pack $w.mb -side left
}

# SetupNetworkSync --
#
# Setup the network synchronization
#
# Arguments:

proc SetupNetworkSync {} {
    global t option setupNS

    # Check if window already exists
    set w .setns
    if {[winfo exists $w]} {
	wm deiconify $w
	raise $w
	return
    }

    # Initialize variables
    set setupNS(w) $w
    set setupNS(deferred) [lindex $option(network_sync) 0]
    set setupNS(disconnected) [lindex $option(network_sync) 1]
    set setupNS(runcmd) [lindex $option(network_sync) 2]
    set setupNS(cmd) [lindex $option(network_sync) 3]

    # Create window
    toplevel $w -bd 5 -class TkRat
    wm title $w $t(setup_netsync)

    checkbutton $w.cmd -variable setupNS(runcmd) -text $t(run_command)
    entry $w.cmdentry -textvariable setupNS(cmd)
    grid $w.cmd $w.cmdentry -sticky w

    checkbutton $w.def -variable setupNS(deferred) -text $t(send_deferred)
    grid $w.def - -sticky w

    checkbutton $w.dis -variable setupNS(disconnected) -text $t(sync_dis)
    grid $w.dis -  -sticky w

    frame $w.f
    button $w.f.ok -text $t(ok) -default active -command {
	destroy $setupNS(w)
	set option(network_sync) [list $setupNS(deferred) \
				       $setupNS(disconnected) \
				       $setupNS(runcmd) \
				       $setupNS(cmd)]
	unset setupNS
	SaveOptions
    }
    button $w.f.cancel -text $t(cancel) \
	    -command {destroy $setupNS(w); unset setupNS}
    pack $w.f.ok $w.f.cancel -side left -expand 1
    grid $w.f - -sticky nsew
}

# SelectFont --
#
# Show font selection
#
# Arguments:
# f - font to select
# w - window to build

proc SelectFont {f w} {
    global pref fixedNormFont t b prefFontLabel

    set d [ConvertFontToText $pref(opt,$f)]

    set prefFontLabel($f) $w.l
    frame $w
    label $w.l -text $d -font [RatCreateFont $pref(opt,$f)] -anchor w
    pack $w.l -side left
    set b($w) pref_$f
    set b($w.l) pref_$f

    button $w.e -text $t(edit)... -command "DoEditFont $f $w.l $w"
    pack $w.e -side right -fill x
}

# ConvertFontToText --
#
# Convert a font specification to text
#
# Arguments:
# s - A font specification

proc ConvertFontToText {s} {
    global t

    if {"components" == [lindex $s 0]} {
	set d [concat [lindex $s 1] [lindex $s 2]]
	if {"bold" == [lindex $s 3]} {
	    set d "$d $t(bold)"
	}
	if {"roman" != [lindex $s 4]} {
	    set d "$d $t(italic)"
	}
	if {[lindex $s 5]} {
	    set d "$d $t(underline)"
	}
	if {[lindex $s 6]} {
	    set d "$d $t(overstrike)"
	}
	return $d
    } else {
	return [lindex $s 1]
    }

}

# DoEditFont --
#
# Show the edit font window
#
# Arguments:
# font	 - Font setting to edit
# l	 - Label to update afterwards
# parent - Parent window

proc DoEditFont {font l parent} {
    global t idCnt pref
    
    set id doEditFont[incr idCnt]
    upvar #0 $id hd

    # Initialization
    set hd(done) 0
    set hd(new_spec) $pref(opt,$font)
    set hd(old_spec) ""
    set hd(font_name) ""

    if {"components" == [lindex $hd(new_spec) 0]} {
	set hd(family) [lindex $hd(new_spec) 1]
	set hd(size) [lindex $hd(new_spec) 2]
	set hd(weight) [lindex $hd(new_spec) 3]
	set hd(slant) [lindex $hd(new_spec) 4]
	set hd(underline) [lindex $hd(new_spec) 5]
	set hd(overstrike) [lindex $hd(new_spec) 6]
	set hd(method) components
    } else {
	set hd(name) [lindex $hd(new_spec) 1]
	set hd(method) name
	set hd(family) Helvetica
	set hd(size) 12
    }

    # Create toplevel
    set w .fontedit
    toplevel $w -class TkRat
    wm title $w $t(edit_font)
    wm transient $w $parent

    # Top label
    label $w.topl -text $t(use_one_method)

    # Specification method frame
    frame $w.s -bd 1 -relief raised
    radiobutton $w.s.select -variable ${id}(method) -value components \
	    -command "UpdateFontSpec $id components"
    label $w.s.fl -text $t(family):
    set m $w.s.family.m
    menubutton $w.s.family -bd 1 -relief raised -indicatoron 1 -menu $m \
	    -textvariable ${id}(family) -width 15
    menu $m -tearoff 0
    if {![info exists pref(families)]} {
	set pref(families) [lsort -dictionary [font families]]
    }
    foreach f $pref(families) {
	$m add command -label $f -command \
		"set ${id}(family) [list $f]; UpdateFontSpec $id components"
    }
    FixMenu $m
    label $w.s.sl -text "  $t(size):"
    set m $w.s.size.m
    menubutton $w.s.size -bd 1 -relief raised -indicatoron 1 -menu $m \
	    -textvariable ${id}(size) -width 3
    menu $m -tearoff 0
    foreach s {4 5 6 7 8 9 10 11 12 13 14 15 16 18 20 22 24 26 30 36} {
	$m add command -label $s -command \
		"set ${id}(size) $s; UpdateFontSpec $id components"
    }
    checkbutton $w.s.weight -text "$t(bold) " -onvalue bold -offvalue normal \
	    -variable ${id}(weight) -command "UpdateFontSpec $id components"
    checkbutton $w.s.italic -text "$t(italic) " -onvalue italic -offvalue roman\
	    -variable ${id}(slant) -command "UpdateFontSpec $id components"
    checkbutton $w.s.underline -text "$t(underline) " \
	    -variable ${id}(underline) -command "UpdateFontSpec $id components"
    checkbutton $w.s.overstrike -text $t(overstrike) \
	    -variable ${id}(overstrike) -command "UpdateFontSpec $id components"

    pack $w.s.select \
	 $w.s.fl $w.s.family \
	 $w.s.sl $w.s.size \
	 $w.s.weight \
	 $w.s.italic \
	 $w.s.underline \
	 $w.s.overstrike -side left -pady 2

    # Name method frame
    frame $w.n -bd 1 -relief raised 
    radiobutton $w.n.select -variable ${id}(method) -value name \
	    -command "UpdateFontSpec $id name"
    label $w.n.l -text $t(name):
    entry $w.n.e -width 20 -textvariable ${id}(name)
    set hd(updateButton) $w.n.set
    button $w.n.set -text $t(update) -command "UpdateFontSpec $id name" -bd 1
    pack $w.n.select \
	 $w.n.l \
	 $w.n.e \
	 $w.n.set -side left -pady 2
    trace variable hd(name) w "UpdateFontUpdateButton $id"
    UpdateFontUpdateButton $id

    # Sample text
    message $w.sample -text $t(ratatosk) -aspect 200 -justify left
    set hd(sample) $w.sample

    # Buttons
    OkButtons $w $t(ok) $t(cancel) "set ${id}(done)"
    set hd(okbutton) $w.buttons.ok

    # Pack things
    pack $w.topl \
	 $w.s \
	 $w.n -side top -fill x -pady 2 -padx 2
    pack $w.buttons -side bottom -fill x -pady 2 -padx 2
    pack $w.sample -fill x -pady 2 -padx 2

    # Bindings
    bind $w.n.e <Tab> "UpdateFontSpec $id name"
    bind $w.n.e <Return> "UpdateFontSpec $id name; break"

    # Update sample font
    UpdateFont $id

    # Show window and wait for completion
    Place $w editFont
    ModalGrab $w
    pack propagate $w 0
    tkwait variable ${id}(done)

    # Finalization
    RecordPos $w editFont
    destroy $w
    set pref(opt,$font) $hd(old_spec)
    if {"" != $hd(font_name)} {
	font delete $hd(font_name)
    }
    unset hd
    $l configure -text [ConvertFontToText $pref(opt,$font)] \
	    -font [RatCreateFont $pref(opt,$font)]
}

# UpdateFontUpdateButton --
#
# Set state of the update button
#
# Arguments:
# handler - Handler of font window
# args    - Possibly standard trace args

proc UpdateFontUpdateButton {handler args} {
    upvar #0 $handler hd

    if {"" != $hd(name) && "name" == $hd(method)} {
	set state normal
    } else {
	set state disabled
    }
    $hd(updateButton) configure -state $state
}

# UpdateFontSpec --
#
# Update the shown font
#
# Arguments:
# handler - Handler of font window
# method  - which method to use

proc UpdateFontSpec {handler method} {
    upvar #0 $handler hd

    if {"components" == $method} {
	set hd(new_spec) [list components $hd(family) $hd(size) $hd(weight) \
				    $hd(slant) $hd(underline) $hd(overstrike)]
	set hd(method) components
    } else {
	set hd(new_spec) [list name $hd(name)]
	set hd(method) name
    }
    UpdateFont $handler
}

# UpdateFont --
#
# Update the sample text
#
# Arguments:
# handler - Handler of font window

proc UpdateFont {handler} {
    upvar #0 $handler hd
    global t

    if {"$hd(new_spec)" == "$hd(old_spec)"} {
	return
    }
    if {[lindex $hd(new_spec) 0] == "components"} {
	if {"" == $hd(font_name)} {
	    set op create
	    set hd(font_name) fontedit
	} else {
	    set op configure
	}
	font $op $hd(font_name) \
		-family [lindex $hd(new_spec) 1] \
		-size -[lindex $hd(new_spec) 2] \
		-weight [lindex $hd(new_spec) 3] \
		-slant [lindex $hd(new_spec) 4] \
		-underline [lindex $hd(new_spec) 5] \
		-overstrike [lindex $hd(new_spec) 6]
	set fn $hd(font_name)
    } else {
	set fn [lindex $hd(new_spec) 1]
    }

    set hd(old_spec) $hd(new_spec)

    if {[catch {$hd(sample) configure -font $fn} err]} {
	set okstatus disabled
	set msg $t(invalid_font)
	set aspect 1000
	$hd(sample) configure -font fixed
    } else {
	set okstatus normal
	set msg $t(ratatosk)
	set aspect 200
    }
    $hd(sample) configure -text $msg -aspect $aspect
    $hd(okbutton) configure -state $okstatus
}

# SetupRole --
#
# Setup the roles buttons
#
# Arguments:
# w - Window to build

proc SetupRole {w} {
    global t pref option

    frame $w

    # Find max length of labels
    set l1 [string length $t(set_as_default)]
    set l2 [string length $t(delete)]
    if {$l1 > $l2} {
	set l $l1
    } else {
	set l $l2
    }

    # Get role id and check for enablement
    set id [lindex [split $pref(rolePrefix) ,] 0]
    if {$id == $option(default_role)} {
	set s disabled
    } else {
	set s normal
    }
    button $w.default -text $t(set_as_default) -pady 0 -width $l -state $s \
	    -command "\
	set option(default_role) $id; \
	$w.default configure -state disabled; \
	$w.delete configure -state disabled; \
	PrefPopulateRoles; \
	$pref(tree) redraw; \
	SaveOptions \
    "
    button $w.delete -text $t(delete) -pady 0 -width $l -state $s \
	    -command "PrefDeleteRole $id"
    pack $w.default $w.delete -side top -anchor w -pady 5
}

# PrefDeleteRole --
#
# Delete a role
#
# Arguments:
# id - role id

proc PrefDeleteRole {id} {
    global option pref vFolderDef t

    set fdls [VFoldersUsesRole $id]
    if {"" != $fdls} {
	Popup "$t(cant_delete_role_used): $fdls"
	return
    }
    set value [RatDialog $pref(pane) $t(delete) $t(are_you_sure_delete_role) \
	    {} 1 $t(delete) $t(cancel)]
    if {1 == $value} {
	return
    }
    foreach v [array names option $id,*] {
	unset option($v)
    }
    set i [lsearch -exact $option(roles) $id]
    set option(roles) [lreplace $option(roles) $i $i]
    set pref(selected) ""
    foreach c [winfo children $pref(pane)] {
	destroy $c
    }
    PrefPopulateRoles
    $pref(tree) redraw
    SaveOptions
}

# PrefTraceProc --
#
# Preferences tracing procedure
#
# Arguments:
# name1, name2 - Name of variable changed
# op           - Operation

proc PrefTraceProc {name1 name2 op} {
    global pref option

    if {!$pref(traceChanges)} {return}

    if {[regexp {^opt,} $name2]} {
	set state disabled
	foreach v $pref(vars,$pref(lastPref)) {
	    if {$option($v) != $pref(opt,$v)} {
		set state normal
		break
	    }
	}
	foreach but $pref(traceButtons) {
	    $but configure -state $state
	}
    }
}

# PrefPopulateRoles --
#
# Populate the roles branch in the preferences tree
#
# Arguments:

proc PrefPopulateRoles {} {
    global pref option t
    
    $pref(rnode) clear
    foreach r $option(roles) {
	set name $option($r,name)
	if {$r == $option(default_role)} {
	    set name "$name ($t(default))"
	}
	set node [$pref(rnode) add folder -label $name -state closed \
		-id [list role $r roles]]
	foreach p {address sending pgp advanced} {
	    $node add leaf -label $t($p) -id [list role $r roles,$p]
	}
    }
    if {"role" == [lindex $pref(selected) 0]} {
	$pref(tree) select $pref(selected)
    }
}

# PrefCreateRole --
#
# Create a new role
#
# Arguments:

proc PrefCreateRole {} {
    global pref option t

    foreach rid $option(roles) {
	lappend ids [string range $rid 1 end]
    }
    set id "r[expr {[lindex [lsort -integer $ids] end]+1}]"
    foreach vn [array names option $option(default_role),*] {
	set v [lindex [split $vn ,] 1]
	set option($id,$v) $option($vn)
    }
    set option($id,name) $t(new_role)
    lappend option(roles) $id
    PrefSortRoles
    SaveOptions

    PrefPopulateRoles
    $pref(tree) select [list role $id roles]
    $pref(tree) redraw
    PrefSelectPane [list role $id roles]
}

# PrefSortRoles --
#
# Sort the list of roles. This function updates the option(roles) list
#
# Arguments:

proc PrefSortRoles {} {
    global option

    set option(roles) [lsort -command PrefCmpRoles $option(roles)]
}
proc PrefCmpRoles {a b} {
    global option

    return [string compare $option($a,name) $option($b,name)]
}

# SetupDefaultSave --
#
# Setup the default save folder
#
# Arguments:
# w - Window to build

proc SetupDefaultSave {w} {
    global t pref option b

    menubutton $w -textvariable pref(text,save_outgoing) -indicatoron 1 \
	    -relief raised -menu $w.m -pady 1
    menu $w.m -postcommand "PopulateDefaultSave $w.m"
    if {"" == $pref(opt,$pref(rolePrefix)save_outgoing)} {
	SelectDefaultSave {}
    } else {
	SelectDefaultSave $pref(opt,$pref(rolePrefix)save_outgoing)
    }
    set b($w) pref_save_outgoing
}

proc PopulateDefaultSave {m} {
    global t

    $m delete 0 end
    VFolderBuildMenu $m 0 "SelectDefaultSave" 1
    $m add command -label "-- $t(none) --" -command {SelectDefaultSave ""}
    FixMenu $m
}

proc SelectDefaultSave {id} {
    global t pref vFolderDef

    set pref(opt,$pref(rolePrefix)save_outgoing) $id
    if {"" == $id} {
	set pref(text,save_outgoing) "-- $t(none) --"
    } else {
	set pref(text,save_outgoing) [lindex $vFolderDef($id) 0]
    }
}


# SetupFontSizeButtons --
#
# Setup the fontsize buttons
#
# Arguments:
# w - Window to build

proc SetupFontSizeButtons {w} {
    global t

    frame $w

    set l1 [string length $t(increase_font_size)]
    set l2 [string length $t(decrease_font_size)]
    if {$l1 > $l2} {
	set l $l1
    } else {
	set l $l2
    }
    button $w.inc -width $l -text $t(increase_font_size) -pady 0 \
	-command {ChangeFontSize 1}
    button $w.dec -width $l -text $t(decrease_font_size) -pady 0 \
	-command {ChangeFontSize -1}
    pack $w.inc -side top -anchor w
    pack $w.dec -side bottom -anchor w
}

# ChangeFontSize --
#
# Changes the font size of all defined fonts where it can be
# changed (i.e. not named fonts)
#
# Arguments:
# delta - Amount to change size with

proc ChangeFontSize {delta} {
    global pref prefFontLabel

    foreach f {prop_big prop_norm prop_light fixed_norm fixed_bold \
		   watcher_font fixed_italic prop_big} {
	if {"components" == [lindex $pref(opt,$f) 0]} {
	    set ns [expr [lindex $pref(opt,$f) 2] + $delta]
	    set pref(opt,$f) [lreplace $pref(opt,$f) 2 2 $ns]
	    $prefFontLabel($f) configure \
		-text [ConvertFontToText $pref(opt,$f)] \
		-font [RatCreateFont $pref(opt,$f)]
	}
    }
}

# CheckSameSendingPrefs --
#
# Checks if all roles uses the same sending preferences as the ones
# currently being shown. If not show a dialog and give teh user the
# option to abort
#
# Arguments:
# rp - role we are currently editing

proc CheckSameSendingPrefs {rp} {
    global option pref t
    
    set differs 0
    foreach r $option(roles) {
	foreach v {sendprot smtp_hosts validate_cert sendprog sendprog_8bit} {
	    if {$option(${r},$v) != $pref(opt,${rp}$v)} {
		set differs 1
		break
	    }
	}
    }

    if {1 == $differs} {
	return [RatDialog .pref $t(send_settings_differs) \
		    $t(roles_use_other_send_settings) {} \
		    1 $t(continue) $t(cancel)]
    } else {
	return 0
    }
}

# PrefSetupSignAs --
#
# Draw the sign-as parts of the preference dialog.
#
# Arguments:
# w - Window to build

proc PrefSetupSignAs {w} {
    global pref

    SetupSignAsWidget $w pref(opt,$pref(rolePrefix)sign_as)
}

# SetupCheckDicts --
#
# Setup the check dictionaries stuff
#
# Arguments:
# w - Window to build

proc SetupCheckDicts {w} {
    global pref b

    frame $w -bd 2
    scrollbar $w.scroll \
        -bd 1 \
        -highlightthickness 0 \
        -command "$w.list yview"
    listbox $w.list \
        -yscroll "$w.scroll set" \
        -bd 1 \
        -exportselection false \
        -highlightthickness 0 \
        -selectmode multiple
    pack $w.scroll -side right -fill y
    pack $w.list -expand 1 -fill both

    set pref(list,auto_dicts) $w.list
    PrefPopulateCheckDicts
    $w.list configure -height 10

    bind $w.list <<ListboxSelect>> {CheckDictsSelect %W}
    set b($w.list) pref_auto_dicts
}

proc PrefPopulateCheckDicts {} {
    global pref

    $pref(list,auto_dicts) delete 0 end
    foreach l $pref(dictionaries) {
        $pref(list,auto_dicts) insert end [string totitle $l]
        if {-1 != [lsearch -exact $pref(opt,auto_dicts) $l]
            || 0 == [llength $pref(opt,auto_dicts)]} {
            $pref(list,auto_dicts) selection set end
        }
    }
}

proc CheckDictsSelect {w} {
    global pref

    set n {}

    foreach s [$w curselection] {
        lappend n [lindex $pref(dictionaries) $s]
    }
    set pref(opt,auto_dicts) $n
}

# SetupSpellPath --
#
# Setup the spell checker path entry
#
# Arguments:
# w - Window to build

proc SetupSpellPath {w} {
    global pref b checkmark_img

    frame $w

    entry $w.entry -textvariable pref(opt,spell_path)
    label $w.status -image $checkmark_img

    pack $w.status -side right -fill y
    pack $w.entry -fill x -pady 2

    set b($w.entry) pref_spell_path
    bind $w.entry <Map> [list UpdateSpellPath $w.status 0]
    bind $w.entry <FocusOut> [list UpdateSpellPath $w.status 0]
    UpdateSpellPath $w.status 1
}

# UpdateSpellPath --
#
# Check which dictionaries the spelling-checker provides and update
# teh display accordingly
#
# Arguments:
# but - Button to update

proc UpdateSpellPath {lab force } {
    global t pref option b checkmark_img error_img

    # Ignore if unchanged
    if {$option(spell_path) == $pref(opt,spell_path) && !$force} {
        return
    }

    set option(spell_path) $pref(opt,spell_path)
    set spell_dictionaries [list [list auto $t(auto)]]
    set pref(dictionaries) [lsort [rat_spellutil::get_dicts 1]]
    foreach l $pref(dictionaries) {
        lappend spell_dictionaries [list $l [string totitle $l]]
    }
    PrefPopulateOptionsMenu def_spell_dict $spell_dictionaries
    PrefPopulateCheckDicts

    if {0 == [llength $pref(dictionaries)]} {
        if {"" == [rat_spellutil::get_cmd]} {
            set err no_spell
        } else {
            set err no_dictionaries
        }
        Popup $t($err) [winfo toplevel $lab]
        set b($lab) $err
        $lab configure -image $error_img
    } else {
        $lab configure -image $checkmark_img
        catch {unset b($lab)}
    }
}
